/***************************************************************************
GHEAR Server Framework.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.ghear;

import java.io.*;
import java.net.*;
import java.util.*;
import com.andthehorseyourodeinon.utils.*;

/** This is the main service thread Object. It's called by Server whenever an HTTP request comes in
on its designated port.

The HTTP that this talks is not really correct, since it just claims to be the
same version of HTTP that was used for the request. A future version should address this.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Server.java,v 1.1 2000/05/16 22:41:13 samuelg Exp $
*/
class HttpService extends Thread {
	
	private Socket socket;
	private BufferedReader in;
	private PrintWriter out;
	private Metrics metrics;
	private ServiceDispatcher dispatcher;
	private Properties options;
	
	/**
	Create a new HttpService thread.
	@param theSock the Socket on which the request was made.
	@param d the ServiceDispatcher which will be interrogated for Service information
	@param rm the global collection of RunMetrics
	@param p the global Properties
	**/
	public HttpService(Socket theSock, ServiceDispatcher d, Metrics rm, Properties p) throws IOException
		{
		socket = theSock;
		metrics = rm;
		dispatcher = d;
		options = p;
		in = new BufferedReader(new InputStreamReader( socket.getInputStream()));
		out = new PrintWriter( new BufferedWriter(
					new OutputStreamWriter(
					socket.getOutputStream())), true);
		start(); // calls run()
		}
		
	/** Actually handle the request */
	public void run()
		{
		metrics.start(dispatcher.getName() + " Reporting");

		ParameterVector pv;
		InetAddress theAddress = socket.getInetAddress();
		int thePort = socket.getPort();
		try
			{
			pv = new ParameterVector(in.readLine());
			String httpVersion = (String)pv.lastElement();
			if (pv.stringAt(1).equalsIgnoreCase("/stats"))
				{
				Response r = new Response(200,metrics.showAllMetrics());
				sendResponse(r, httpVersion);
				}
			else if (pv.stringAt(1).equalsIgnoreCase("/readme"))
				{
				Response r = new Response(200,"readme.html", true);
				sendFileResponse(r, httpVersion);
				}
			else if (pv.stringAt(1).equalsIgnoreCase("/license"))
				{
				out.print(httpVersion+
					" 302 Moved Temporarily\n" + 
					"location: http://www.gnu.org/copyleft/gpl.html\n\n");
				}
			else
				{
				Response r = dispatcher.getDispatchReport(pv);
				if (r.isFile())
					{
					sendFileResponse(r, httpVersion);
					}
				else
					{
					sendResponse(r, httpVersion);
					}
				}
				
			out.flush();
			out.close();
			in.close();
			socket.close();
			metrics.stop(dispatcher.getName() + " Reporting");
			Thread.currentThread().stop();
			}
		catch (Exception e)
			{
			Logger.log("Service errror: "+e.getMessage());
			e.printStackTrace();
			}
		finally
			{
			try
				{
				socket.close();
				}
			catch (Exception fc)
				{
				Logger.log("Service errror: "+fc.getMessage());
				fc.printStackTrace();

				}
			}
		}
	
	/** Send a response, wrapped in the correct HTTP headers.
	@param gr the Response object containing the response
	@param httpVersion the version of HTTP that was used in the request.
	*/	
	public void sendResponse(Response gr, String httpVersion)
		{
		int wrapperLen = 0;
		Logger.debug(Logger.BORE,"Returning "+gr.getCode()+" "+HTTPUtils.statusMessage(gr.getCode()));

		if (httpVersion == null)
			{
			out.println("HTTP/1.1 "+gr.getCode()+" "+HTTPUtils.statusMessage(gr.getCode()));
			}
		else
			{
			out.println(httpVersion+" "+gr.getCode()+" "+HTTPUtils.statusMessage(gr.getCode()));
			}
		out.println("Content-type: text/html");
		out.println("Date: "+StringUtils.createRFC1123DateString());
		if (options.getProperty("usestdheader","true").equalsIgnoreCase("true"))
			{
			wrapperLen += FileCache.length("stdheader.html");
			}
		if (options.getProperty("usestdfooter","true").equalsIgnoreCase("true"))
			{
			wrapperLen += FileCache.length("stdfooter.html");
			}
		out.println("Content-length: "+gr.getLen()+wrapperLen);
		out.println("Connection: close\n");
		if (options.getProperty("usestdheader","true").equalsIgnoreCase("true"))
			{
			out.print(FileCache.toString("stdheader.html"));
			}
		out.print(gr.getText());
		if (options.getProperty("usestdfooter","true").equalsIgnoreCase("true"))
			{
			out.print(FileCache.toString("stdfooter.html"));
			}
		}

	/** Send a file back as a response, wrapped in the correct HTTP headers.
	@param gr the Response object containing the response filespec.
	@param httpVersion the version of HTTP that was used in the request.
	*/	
	public void sendFileResponse(Response gr, String httpVersion)
		{
		Logger.debug(Logger.BORE,"Returning file "+gr.getText());

		sendResponse(new Response(200, FileCache.toString(gr.getText())), httpVersion);
		}

	}


/** This is the main Server code.

This reads in the config files, initializes the runtime metrics, initializes the ServiceDispatcher,
and starts a listener thread to handle incoming requests.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Server.java,v 1.1 2000/05/16 22:41:13 samuelg Exp $
*/
public class Server {
	public static final String VERSION = "0.5";
	
	public static void main(String args[])
		{
		String configFilespec;
		ServerSocket serverSocket = null;
		Metrics mets = new Metrics();
		ServiceDispatcher sd;
		
		mets.start(" Initialization of Server");
		if (args.length < 1)
			{
			System.out.println("Usage: Server [config file]");
			configFilespec = "ghear.conf";
			}
		else
			{
			configFilespec = args[0];
			}
		Properties options = new Properties();
		try
			{
			options.load(new FileInputStream(configFilespec));
			}
		catch (IOException fnfe)
			{
			Logger.log("Can't read properties from "+ configFilespec);
			}
		if (!options.getProperty("write-log","false").equalsIgnoreCase("false"))
			{
			Logger.setFile();
			}
		Logger.verbose();
		Logger.log("Starting GHEAR version "+VERSION+ ".");
		Logger.log("GHEAR is the Generic HTTP-Enabled Application Responder. It's pronounced \"Fear\" for a Reason!");
		Logger.log("Using configuration from "+configFilespec+".");
		Logger.log("Running Responder on socket "+options.getProperty("port","16670"));
		Logger.setDebugLevel(Integer.parseInt(options.getProperty("verbose",Integer.toString(Logger.SILENT))));

		try
			{
			Logger.debug(Logger.BORE,"Getting class for name from config file");
			Class theDispatcher = Class.forName(options.getProperty("dispatch-service","ghear.ServiceDispatcher"));
			Logger.debug(Logger.BORE,"Created Class");
			sd = (ServiceDispatcher)theDispatcher.newInstance();
			
			Logger.debug(Logger.BORE,"Successfully created a new instance of "+
				options.getProperty("dispatch-service","ghear.ServiceDispatcher"));
			
			sd.initDispatcher(options, mets);
			sd.initServices();

			try
				{
				serverSocket =
					new ServerSocket(Integer.parseInt(options.getProperty("port","16670")));
				mets.stop(" Initialization of Server");
				while (true)
					{
					Socket socket = serverSocket.accept();
					mets.start("Responder Main Thread");
					try
						{
						Logger.debug(Logger.BORE,"Responder Thread got a request.");
						new HttpService(socket, sd, mets, options);
						}
					catch (IOException ioe)
						{
						Logger.debug(Logger.ERROR,"Responder Service failed. Closing socket,");
						socket.close();
						}
					mets.stop("Responder Main Thread");
					}
				}
			catch (Exception serverException)
				{
				Logger.log("Exception: " + serverException.getMessage());
				serverException.printStackTrace();
				}
			}
		catch (Exception ci)
			{
			Logger.debug(Logger.FATAL,"Can't create dispatcher instance for "+
				options.getProperty("dispatch-service","ghear.ServiceDispatcher"));
			Logger.close();
			System.exit(1);
			}
			
		Logger.close();
		}
 
}

