/***************************************************************************
ServiceDispatcher Abstract Class.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/package com.andthehorseyourodeinon.ghear;

import java.io.*;
import java.net.*;
import java.util.*;
import com.andthehorseyourodeinon.utils.*;

/** This is an abstract class describing a ServiceDispatcher.
A ServiceDispatcher is a controller for a collection of Services.
It also handles reporting for the Services as a whole as well as
individually.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: ServiceDispatcher.java,v 1.1 2000/05/16 22:41:13 samuelg Exp $
*/
public abstract class ServiceDispatcher {

	protected Properties serverOptions;
	protected Properties options;
	protected Hashtable services;
	protected String dispatcherName;
	protected Metrics mets;

	public ServiceDispatcher()
		{
		services = new Hashtable();
		dispatcherName = "";
		}

	/* Override me with something that will
	   initialize the Dispatcher
	*/ 
	public void initDispatcher(Properties serverOptions, Metrics metrics)
		{
		this.serverOptions = serverOptions;
		this.mets = metrics;
		}
		
	/* Override me with something that will
	   initialize services, and store them in a hashtable
	*/ 
	public void initServices()
		{
		}
		
	/* Override me with something that will
	   add a new service, and store it in the services hashtable, or just
	   leave as is.
	*/ 
	public void addService(String identifier, Service newService)
		{
		services.put(identifier, newService);
		}
		
	/** Get a list of Service identifiers.
	@return a Vector containing Strings, each identifying a specific Service
	*/		
	public Vector serviceList()
		{
		Vector v = new Vector();
		Enumeration e = services.keys();
		while (e.hasMoreElements())
			{
			v.addElement((String)e.nextElement());
			}
		return v;
		}
		
	/** Determines if a Service is known by the ServiceDispatcher.
	@param identifier the Service in question.
	@return true if the Service is known.
	*/		
	public boolean hasService(String identifier)
		{
		if (services.containsKey(identifier))
			{
			return true;
			}
		else
			{
			return false;
			}
		}
		
	/** Get the Service name for a specific Service.
	@param identifier the Service identifier
	@return name of the Service
	*/		
	public String serviceName(String identifier)
		{
		if (hasService(identifier))
			{
			return ((Service)services.get(identifier)).getServiceName();
			}
		else
			{
			return "Unknown Service";
			}
		}

	/** Get a Service status string from a specific Service.
	@param identifier the Service to be interrogated
	@return a status String from the Service
	*/		
	public String serviceStatus(String identifier)
		{
		if (hasService(identifier))
			{
			return ((Service)services.get(identifier)).getServiceStatus();
			}
		else
			{
			return "Unknown Service";
			}
		}

	/** Get a Service report from a specific Service.
	@param identifier the Service to be interrogated
	@param reportType a String identifying the report requested.
	@return a Response containing the report from the Service
	*/		
	public Response serviceReport(String identifier, String reportType)
		{
		if (hasService(identifier))
			{
			return new Response(HttpURLConnection.HTTP_OK,
				((Service)services.get(identifier)).getServiceReport(reportType));
			}
		else
			{
			return new Response(HttpURLConnection.HTTP_NOT_FOUND,
				"Unknown Service");
			}

		}
	
	/** Get the name of the ServiceDispatcher.
	*/
	public String getName()
		{
		return dispatcherName;
		}

	/** Get the number of services being handled.
	@return integer number of service
	*/
	public int getServiceCount()
		{
		return services.size();
		}

	/* This is the portion that handles HTTP requests. The path of the
	   request is handed over as a ParameterVector. It can be parsed however makes
	   sense.
	   
	   One example for simple, single-level requests would look like:
	   
		if (((String)pv.stringAt(1)).toLowerCase().startsWith("request1"))
			{
			return new Response(HttpURLConnection.HTTP_OK,"This is a request for a report of type 1");
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("request2"))
			{
			return new Response(HttpURLConnection.HTTP_OK,"This is a request for a report of type 2");
			}
		else
			{
			return new Response(HttpURLConnection.HTTP_NOT_FOUND,"request "+reportRequest+" not found.");
			}
	
	   so "http://server:port/request1" would return "This is a request for a report of type 1".
	   	
	*/ 
	public Response getDispatchReport(ParameterVector pv)
		{
		return new Response("");
		}

}

