/***************************************************************************
Search Engine Monitor Service
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.netgrep;

import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.*;
import com.andthehorseyourodeinon.ghear.*;

/**  Abstract class for a Search Engine Monitor (or Tracker) Service.
A general Search Engine Monitor will query a serach engine for specific
terms, and keep track of the results and the order of the results. A
Tracker has the additional functionality of tracking the position of a
given site for particular keywords.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: SearchEngineMonitorService.java,v 1.2 2000/05/22 04:53:44 samuelg Exp $
*/
public abstract class SearchEngineMonitorService extends
	com.andthehorseyourodeinon.ghear.Service {

	protected String searchTerm;
	protected String searchHost;
	protected String queryString;
	protected String postFix;
	protected String soughtSite;
	protected int depth;
	protected Hashtable urls;
	protected boolean hasNewUrls = false;
	protected long updateFrequency;
	protected Calendar lastAccessed;
	protected Metrics mets;
	protected String theRequest;
	protected boolean firstTime;

	/**
	@param options Properties general to all the search engine Monitors and Trackers.
	@param mets Metrics for all the search engine Monitors and Trackers.
	@param searchTerm a String containing the search terms to use against the search engine.
	@param searchHost the base address of the URL for the search engine.
	@param queryString a template for a query to the search engine.
	@param postFix a template for anything on the URL after the search terms.
	@param site the base URL (e.g., www.site.com) for a site to be tracked.
	@param depth the number of results from the search engine to consider.
	*/
	public SearchEngineMonitorService(Properties options, Metrics mets,String searchTerm,
		String searchHost, String queryString, String postFix, String site, int depth)
		{
		urls = new Hashtable();
		this.searchTerm = searchTerm;
		this.searchHost = searchHost;
		this.queryString = queryString;
		this.postFix = postFix;
		this.options = options;
		soughtSite = site;
		this.depth = depth;
		this.mets = mets;
		lastAccessed = Calendar.getInstance();
		updateFrequency = Integer.parseInt(options.getProperty("refresh", "86400000")); /* run an update daily by default */
		Logger.debug(Logger.BORE,"Starting new thread looking for "+searchTerm);
		firstTime = true;
		grabUrlList();
		start();
		}
	
	/**
	Main thread. Sleeps for a given period of time, then updates the query results
	*/
	public void run()
		{
		while (true)
			{
			try
				{
				sleep(updateFrequency);
				}
			catch (InterruptedException ie)
				{
				Logger.debug(Logger.ERROR,"Thread for "+searchTerm+" on "+searchHost+
					" interrupted!");
				}
			Logger.debug(Logger.BORE,"Thread for "+searchTerm+" on "+searchHost+" awake.");
			refreshUrlList();
			}
		}
		

	/** Used in the initialization of the Monitor or Tracker to populate the list of
	URLs for a given query
	*/
	public void grabUrlList()
		{
		refreshUrlList(false);
		}
	/** Used to refresh the list of URLs for all subsequent queries
	*/
	public void refreshUrlList()
		{
		refreshUrlList(true);
		}
	
	/** This code should be overridden for the specific task of Monitoring or Tracking
	*/
	public void refreshUrlList(boolean reportNewUrls)
		{
		}

	/** Is a given URL in the list of results?
	@param theUrl String representation of the URL
	@return true if URL is in the list
	*/
	public boolean hasUrl(String theUrl)
		{
		if (urls.containsKey(theUrl))
			{
			return true;
			}
		return false;
		}
		
	/**
	Get a specific result URL.
	@param theUrl String representation of the URL
	@return a SearchEngineRecord of the URL requested, or null if it's not in the
	list of results.
	*/
	public SearchEngineRecord getUrl(String theUrl)
		{
		if (hasUrl(theUrl))
			{
			return (SearchEngineRecord)urls.get(theUrl);
			}
		else
			{
			return null;
			}
		}

	/**
	Get all result URLs modified after a given time.
	@param cal Calendar containing the threshhold time
	@return a Vector of SearchEngineRecords of URLs modified after specified time.
	*/
	public Vector getUrlsModifiedAfter(Calendar cal)
		{
		Vector ret = new Vector();
		Enumeration e = urls.keys();
		while (e.hasMoreElements())
			{
			SearchEngineRecord theUrl = (SearchEngineRecord)e.nextElement();
			if ((theUrl.lastUpdated()).after(cal))
				{
				ret.addElement(theUrl);
				}
			}
		return ret;
		}


	/**
	Get all result URLs.
	@return a Vector of SearchEngineRecords of URLs.
	*/
	public Vector getUrls()
		{
		Vector ret = new Vector();
		Enumeration e = urls.keys();
		while (e.hasMoreElements())
			{
			ret.addElement((SearchEngineRecord)urls.get(e.nextElement()));
			}
		return ret;
		}


	/**
	Get all result URLs created after a given time.
	@param cal Calendar containing the threshhold time
	@return a Vector of SearchEngineRecords of URLs created after specified time.
	*/
	public Vector getUrlsCreatedAfter(Calendar cal)
		{
		Vector ret = new Vector();
		Enumeration e = urls.keys();
		while (e.hasMoreElements())
			{
			SearchEngineRecord theUrl = (SearchEngineRecord)e.nextElement();
			if ((theUrl.created()).after(cal))
				{
				ret.addElement(theUrl);
				}
			}
		return ret;
		}


	/**
	Wrapper method to send a notification email.
	@param subject subject of the email.
	@param text body of the email.
	*/
	public void sendUpdate(String subject, String text)
		{
		mets.start("Email notification");
		Mailer m = new Mailer(options, subject, text);
		mets.stop("Email notification");
		}
				
	public void updateTimestamp()
		{
		lastAccessed = Calendar.getInstance();
		}
		
	public Calendar getTimestamp()
		{
		return lastAccessed;
		}
	
	/** Is there any new data?
	@return true if new URLs exist in the result list
	*/
	public boolean hasNewUrl()
		{
		boolean hadNewUrls = hasNewUrls;
		hasNewUrls = false;
		return hadNewUrls;
		}
		
	public String getSearchString()
		{
		return searchTerm;
		}

	/**
	Override this method
	*/
	public String getSiteString()
		{
		return "";
		}
		
	/**
	Override this method if writing a Tracker
	*/
	public String currentPosition()
		{
		return "";
		}
		
	/**
	Override this method
	*/
	public boolean isMonitor()
		{
		return true;
		}

	public String getSearchHost()
		{
		return searchHost;
		}
		
	/**
	Override this method, writing a Tracker
	*/
	public String history()
		{
		return "";
		}

	public int getSearchDepth()
		{
		return depth;
		}
		
	public int getResultCount()
		{
		return urls.size();
		}
		
	public String getRequest()
		{
		return theRequest;
		}
		
	/**
	Override this method
	*/
	public String getServiceName()
		{
		return "SearchEngineMonitorService";
		}
}
