/***************************************************************************
Message Logging Utility Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.io.*;
import java.util.*;

/** 	The Logger class allows you to have a debug facility that nicely
labels the output with a timestamp, can write to a file, and that
can have the output turned on and off dynamically.

It's equivalent to the Messager class, but can be used statically.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Logger.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class Logger implements Runnable {
        public static final short BORE = 5;
        public static final short INFORM = 4;
        public static final short WARN = 3;
        public static final short ERROR = 2;
        public static final short FATAL = 1;
	public static final short SILENT = 0;
	
	private static final String identifierTag="Logger.debug";

	private static boolean verbose = false;
	private static boolean useFile = false;
	private static int debugLevel = 0;
	private static String fileSpec;
	private static PrintWriter out;
	
	
	private Logger()
		{
		new Thread(this).start();
		}
	
	/**
	Activate this as a single logging thread that won't get garbage collected.
	*/
	public void run()
		{
		try
			{
			wait();
			}
		catch (InterruptedException e)
			{
			}
		}
		
	/**
	Sets logging to a file, closing any previous log files. Remember to call close if you log to a file!
	@param logSpec name of file to write log to.
	*/
	public static void setFile(String logSpec)
		{
		fileSpec = logSpec;
		if (useFile)
			{
			out.close();
			}
		try
			{
			out = new PrintWriter(new
				FileOutputStream(new File(fileSpec)));
			}
		catch (IOException e)
			{
			System.out.println(
				"panic! Can't open "+ fileSpec);
			}
		useFile = true;
		}

	/**
	Sets logging to a unique file, closing any previous log files. Remember to call close if you log to a file!
	@param logSpec name of file to write log to.
	*/
	public static void setFile()
		{
		fileSpec = StringUtils.timestampFilespec("log","txt");
		if (useFile)
			{
			out.close();
			}
		try
			{
			out = new PrintWriter(new
				FileOutputStream(new File(fileSpec)));
			}
		catch (IOException e)
			{
			System.out.println(
				"panic! Can't open "+ fileSpec);
			}
		useFile = true;
		}

	
	/**
	Closes a log file.
	*/
	public static void close()
		{
		if (useFile)
			{
			try
				{
				out.close();
				}
			catch (Exception e)
				{
				System.out.println(
					"panic! Can't close "+ fileSpec);
				}
			
			}
		}
	
	/**
	Disables output from this Messager
	*/
	public static void silent()
		{
		verbose = false;
		}
		
	/**
	Sets the debug level output.
	*/
	public static void setDebugLevel(int level)
		{
		debugLevel = level;
		}
		
	/**
	Enables output from this Messager
	*/
	public static void verbose()
		{
		verbose = true;
		}
	
	/**
	Toggles the output enable state of this Messager. If output was disabled,
	this enables it, and vice versa.
	*/
	public static void toggleMode()
		{
		if (verbose) verbose=false;
		}
		
	/**
	When output is enabled, and the message level is equal to or greater than
	the set debug level, writes a debug line to the console and, if a log file is
	established, to the log file. If output is disabled or the message level is
	less than the debug level, the text is ignored.
	@param level the message level
	@param text the line to output
	*/	
	public static void debug(int level, String text)
		{
		if (level <= debugLevel)
			{
			String routine;
			ByteArrayOutputStream bas = new ByteArrayOutputStream();
			PrintWriter pw = new PrintWriter(bas);
			String levelString = new String();
			
			if (level >= BORE)
				{
				levelString = "BORE";
				}
			if (level == INFORM)
				{
				levelString = "INFORM";
				}
			if (level == WARN)
				{
				levelString = "WARN";
				}
			if (level == ERROR)
				{
				levelString = "ERROR";
				}
			if (level == FATAL)
				{
				levelString = "FATAL";
				}
				
			// This is pretty much an ugly hack, but it tells you who called the
			// debug routine. Note that debug is pretty expensive, in terms of
			// overall performance. But it can be useful!
			try
				{
				throw (new Exception());
				}
			catch (Exception e)
				{
				e.printStackTrace(pw);
				}
				
			pw.flush();
			try
				{
				// tsack trace should be displayed in good old ISO 8859-1 encoding
				routine = bas.toString("iso-8859-1");
				routine = routine.trim();
				
				// Try to cut out the calling routine, and write the debug line.
				if (routine.indexOf(identifierTag) == -1)
					{
					// Hm. We're not in the stack trace. Well, we'll just cope
					// with it.
					log("Debug level "+levelString+": "+text);
					}
				// OK, so we cut everything in the stack trace from the "at" following
				// the identifying tag (i.e., this method's signature) to the newline
				// character after that. Golly, I sure hope the format for a stack trace is
				// consistent across platforms...
				
				int start = routine.indexOf("at",routine.indexOf(identifierTag)) + "at".length();
				int end = routine.indexOf("\n",start);
				if (end == -1)
					{
					end = routine.length();
					}
				
				log(levelString+"-level debug from" +
					routine.substring(start, end) + ": " + text);
				}
			catch (UnsupportedEncodingException uee)
				{
				System.out.println("Can't print BytaArrayOutputString due to encoding request.");
				}
			}

		}
		
	/**
	When output is enabled, writes a line to the console and, if a log file is
	established, to the log file. If output is disabled, the text is ignored.
	@param text the line to output
	*/	
	public static void log(String text)
		{
		if (verbose)
			{
			String prefix = StringUtils.createCompactDateString()+"> ";
			
			System.out.println(prefix+text);
			if (useFile)
				{
				try
					{
					out.println(prefix+text);
					out.flush();
					}
				catch (Exception e)
					{
					System.out.println(prefix+
						"panic! Can't write to "+ fileSpec);
					}
				}
			}
		}


}
