/***************************************************************************
Message Logging  Utility Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.io.*;
import java.util.*;

/**
The Messager class allows you to have a debug facility that nicely
labels the output with a timestamp, can write to a file, and that
can have the output turned on and off dynamically.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Messager.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class Messager {
        public static final short INFORM = 1;
        public static final short WARN = 2;
        public static final short ERROR = 3;
        public static final short FATAL = 4;

	private boolean verbose = false;
	private boolean useFile = false;
	private int debugLevel = 0;
	private String fileSpec;
	private PrintWriter out;
	
	public Messager()
		{
		}
	
	/**
	Constructor that specifies output mode.
	@param isVerbose enables output when true, disables output when false.
	*/ 
	public Messager(boolean isVerbose)
		{
		verbose = isVerbose;
		}
		
	/**
	Constructor that specifies a filename to write log to. This constructor
	assumes output is enabled. Remember to call close if you log to a file!
	@param theLogFile filespec to write log to. File will be clobbered if it exists.
	*/ 
	public Messager(String theLogfile)
		{
		verbose = true;
		fileSpec = theLogfile;
		setFile(theLogfile);
		}
	
	/**
	Sets logging to a file, closing any previous log files. Remember to call close if you log to a file!
	@param logSpec name of file to write log to.
	*/
	public void setFile(String logSpec)
		{
		if (useFile)
			{
			out.close();
			}
		try
			{
			out = new PrintWriter(new
				FileOutputStream(new File(logSpec)));
			}
		catch (IOException e)
			{
			System.out.println(
				"panic! Can't open "+ logSpec);
			}
		useFile = true;
		}
	
	/**
	Closes a log file.
	*/
	public void close()
		{
		if (useFile)
			{
			try
				{
				out.close();
				}
			catch (Exception e)
				{
				System.out.println(
					"panic! Can't close "+ fileSpec);
				}
			
			}
		}
	
	/**
	Disables output from this Messager
	*/
	public void silent()
		{
		verbose = false;
		}
		
	/**
	Sets the debug level output.
	*/
	public void setDebugLevel(int level)
		{
		debugLevel = level;
		}
		
	/**
	Enables output from this Messager
	*/
	public void verbose()
		{
		verbose = true;
		}
	
	/**
	Toggles the output enable state of this Messager. If output was disabled,
	this enables it, and vice versa.
	*/
	public void toggleMode()
		{
		if (verbose) verbose=false;
		}
		
	/**
	When output is enabled, and the message level is equal to or greater than
	the set debug level, writes a debug line to the console and, if a log file is
	established, to the log file. If output is disabled or the message level is
	less than the debug level, the text is ignored.
	@param level the message level
	@param text the line to output
	*/	
	public void debug(int level, String text)
		{
		if (level >= debugLevel)
			{
			String routine;
			ByteArrayOutputStream bas = new ByteArrayOutputStream();
			PrintWriter pw = new PrintWriter(bas);

			// This is pretty much an ugly hack, but it tells you who called the
			// debug routine. Note that debug is pretty expensive, in terms of
			// overall performance. But it can be useful!
			try
				{
				throw (new Exception());
				}
			catch (Exception e)
				{
				e.printStackTrace(pw);
				}
				
			pw.flush();
			try
				{
				// tsack trace should be displayed in good old ISO 8859-1 encoding
				routine = bas.toString("iso-8859-1");
				System.out.println(routine);
				routine = routine.trim();
				// cut out the calling routine, and write the debug line.
				out("(Debug level "+level+" from "+routine.substring(routine.lastIndexOf("at")+2)+") "+text);
				}
			catch (UnsupportedEncodingException uee)
				{
				System.out.println("Can't print BytaArrayOutputString due to encoding request.");
				}
			}

		}
		
	/**
	When output is enabled, writes a line to the console and, if a log file is
	established, to the log file. If output is disabled, the text is ignored.
	@param text the line to output
	*/	
	public void out(String text)
		{
		if (verbose)
			{
			String prefix = "";
			Calendar now = Calendar.getInstance();
			prefix = now.get(Calendar.YEAR) + "-";
			if (now.get(Calendar.MONTH) < 9)
				{
				prefix += "0";
				}
			prefix += (now.get(Calendar.MONTH)+1)+"-";
			if (now.get(Calendar.DAY_OF_MONTH) < 10)
				{
				prefix += "0";
				}
			prefix += now.get(Calendar.DAY_OF_MONTH)+" ";
			if (now.get(Calendar.HOUR_OF_DAY) < 10)
				{
				prefix += "0";
				}
			prefix += now.get(Calendar.HOUR_OF_DAY)+":";
			if (now.get(Calendar.MINUTE) < 10)
				{
				prefix += "0";
				}
			prefix += now.get(Calendar.MINUTE)+":";
			if (now.get(Calendar.SECOND) < 10)
				{
				prefix += "0";
				}
			prefix += now.get(Calendar.SECOND)+"> ";
			
			System.out.println(prefix+text);
			if (useFile)
				{
				try
					{
					out.println(prefix+text);
					out.flush();
					}
				catch (Exception e)
					{
					System.out.println(prefix+
						"panic! Can't write to "+ fileSpec);
					}
				}
			}
		}


}
