/***************************************************************************
Form Storage Data Structure Class.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.util.*;
import java.text.DateFormat;
import java.net.*;
import java.io.*;
import java.text.DecimalFormat;

/** 	PostedForm is a data storage class for non-servlet-based, multi-part
MIME-encoded document uploads.
An uploaded form is stored as a collection of headers (generally
generated by the browser), a collection of content parts, and
a collection of MIME-types (corresponsing to the content parts).
Content parts and their corresponding MIME-types are mapped by a
unique section name.
@see HTTPUtils (specifically the formSubmission and multipartForm methods).
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: PostedForm.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class PostedForm {
	private Hashtable headers;
	private Hashtable content;
	private Hashtable contentType;
	
	public PostedForm()
		{
		headers = new Hashtable();
		content = new Hashtable();
		contentType = new Hashtable();
		}

	/** Headers are stored as name-value pairs.
	@return an Enumeration of the names of headers*/
	public Enumeration headers()
		{
		return headers.keys();
		}

	/** Contents are stored as name-indexed sections.
	@return an Enumeration of the content section names
	*/
	public Enumeration contents()
		{
		return content.keys();
		}
	
	/** Retrieve a named browser header, <I>e.g.</I>, "user_agent".
	@return the header value for a given header name */	
	public String getHeader(String name)
		{
		if (hasHeader(name))
			{
			return (String)headers.get(name);
			}
		else
			{
			return null;
			}
		}
		
	/** Retrieve a named browser header, <I>e.g.</I>, "user_agent".
	Legacy wrapper method for getHeader.
	@return the header value for a given header name */		
	public String getHeaderString(String name)
		{
		return getHeader(name);
		}

	/** Retrieve the type of a named content section.
	@return the MIME-type for a given section name */	
	public String getContentType(String name)
		{
		if (hasContentType(name))
			{
			return (String)contentType.get(name);
			}
		else
			{
			return null;
			}
		}
		
	/** Retrieve a named content section as an array of bytes.
	@return an array of bytes*/	
	public byte[] getContent(String name)
		{
		if (hasContent(name))
			{
			return (byte[])content.get(name);
			}
		else
			{
			return null;
			}
		}
		
	/** Retrieve a named content section as a String.
	@return a String*/	
	public String getContentString(String name)
		{
		if (hasContent(name))
			{
			return new String((byte [])content.get(name));
			}
		else
			{
			return null;
			}
		}
		
	/** add a name-value pair to the collection of headers */
	public void addHeader(String name, String value)
		{
		headers.put(name,value);
		}
		
	/** add a String containing a name-value pair to the collection of headers
	@param unparsed string of the form "name=value" */
	public void addHeader(String unparsed)
		{
		if (unparsed.indexOf(":") == -1)
			{
			return;
			}
		addHeader(unparsed.substring(0,unparsed.indexOf(":")).toLowerCase(),
			unparsed.substring(unparsed.indexOf(":")+1).trim());
		}
	
	/** add a MIME-type to the collection of content-types for a named section */
	public void addContentType(String name, String value)
		{
		contentType.put(name,value);
		}


	/** add a byte array as a named section, assuming a String (ISO-8859-1) MIME-type. */
	public void addContent(String name, byte[] value)
		{
		addContent(name,value,"iso-8859-1");
		}

	/** add a byte array as a named section, using the specified MIME-type. */
	public void addContent(String name, byte[] value, String theContentType)
		{
		content.put(name,value);
		contentType.put(name,theContentType);
		}

	/** determine whether a named header exists */
	public boolean hasHeader(String name)
		{
		if (headers.containsKey(name))
			{
			return true;
			}
		else
			{
			return false;
			}
		}

	/** determine whether a named section has content */
	public boolean hasContent(String name)
		{
		if (content.containsKey(name))
			{
			return true;
			}
		else
			{
			return false;
			}
		}

	/** determine whether a named section has defined a MIME-type */
	public boolean hasContentType(String name)
		{
		if (contentType.containsKey(name))
			{
			return true;
			}
		else
			{
			return false;
			}
		}

}
