/***************************************************************************
Runtime Metrics Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.util.Date;
import com.andthehorseyourodeinon.utils.StringUtils;

/** 	RunMetrics is a class for tracking time. It's mostly useful for
providing and reporting run-time metrics of a process (or section of
code) that is run multiple times. It then can report on the average,
minimum, and maximum runtimes, along with total time used.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: RunMetrics.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class RunMetrics {
	private long	totalTime;
	private long	shortestTime;
	private long	longestTime;
	private long	count;
	private long	start;
	private long	stop;
	private String	metricName;
	

	public RunMetrics()
		{
		initRunMetrics("unnamed metric");
		}

	public RunMetrics(String name)
		{
		initRunMetrics(name);
		}

	private void initRunMetrics(String name)
		{
		totalTime = 0;
		shortestTime = Long.MAX_VALUE;
		longestTime = 0;
		count = 0;
		start = 0;
		stop = 0;
		metricName = name;
		}

	/** set the name of this metric */
	public void setName(String name)
		{
		metricName = name;
		}

	/** get the name of this metric */
	public String getName()
		{
		return metricName;
		}

	/** start the timing cycle */
	public void start()
		{
		start = System.currentTimeMillis();
		}
		
	/** stop the timing cycle. Better-named wrapper for the end method. */
	public void stop()
		{
		end();
		}

	/** stop the timing cycle */
	public void end()
		{
		stop = System.currentTimeMillis();
		update(start, stop);
		}


	/** update the statistics for this metric */
	private void update(long startTime, long endTime)
		{
		long thisTime = endTime - startTime;
		count++;
		totalTime = totalTime + thisTime;
		if (thisTime < shortestTime)
			{
			shortestTime = thisTime;
			}
		if (thisTime > longestTime)
			{
			longestTime = thisTime;
			}
		
		}
		
	/** generate a text report on this metric.*/
	public String reportText()
		{
		if (count == 0)
			{
			return metricName + " has no data.";
			}
		else
			{
			return  metricName+":\nAverage Time: "+
				StringUtils.reasonableTimeUnits(totalTime/count)+
				"\nTotal Time: "+
				StringUtils.reasonableTimeUnits(totalTime) +
				"\nShortest Time: "+
				StringUtils.reasonableTimeUnits(shortestTime)+
				"\nLongest Time: "+
				StringUtils.reasonableTimeUnits(longestTime) +
				"\n";
			}
		}
	
	/** generate an HTML-formetted report on this metric, using a table */
	public String reportHTML()
		{
		if (count == 0)
			{
			return  "<TABLE BORDER=\"1\"><TR><TD COLSPAN=\"2\"><B>"+metricName+
				"</B></TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total samples:</TD><TD>"+
				"No Data Gathered</TD></TR></TABLE>";
			}
		else if (count == 1)
			{
			return  "<TABLE BORDER=\"1\"><TR><TD COLSPAN=\"2\"><B>"+metricName+
				"</B></TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total samples:</TD><TD>"+
				count+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Average Time:</TD><TD>&nbsp;"+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total Time:</TD><TD>"+
				StringUtils.reasonableTimeUnits(totalTime) +
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Shortest Time:</TD><TD>&nbsp;"+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Longest Time:</TD><TD>&nbsp;"+
				"</TD></TR></TABLE>";
			}
		else
			{
			return  "<TABLE BORDER=\"1\"><TR><TD COLSPAN=\"2\"><B>"+metricName+
				"</B></TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total samples:</TD><TD>"+
				count+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Average Time:</TD><TD>"+
				StringUtils.reasonableTimeUnits(totalTime/count)+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total Time:</TD><TD>"+
				StringUtils.reasonableTimeUnits(totalTime) +
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Shortest Time:</TD><TD>"+
				StringUtils.reasonableTimeUnits(shortestTime)+
				"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Longest Time:</TD><TD>"+
				StringUtils.reasonableTimeUnits(longestTime) +
				"</TD></TR></TABLE>";
			}
		}
		
	public String getAvgTime()
		{
		if (count == 0)
			{
			return StringUtils.displayTimeUnits(0, true);
			}
		return StringUtils.displayTimeUnits(totalTime/count, true);
		}
		
	public String getTotalTime()
		{
		return StringUtils.displayTimeUnits(totalTime, true);
		}
		
	public String getShortestTime()
		{
		return StringUtils.displayTimeUnits(shortestTime, true);
		}

	public String getLongestTime()
		{
		return StringUtils.displayTimeUnits(longestTime, true);
		}
		
	public String getTotalSamples()
		{
		return ""+count;
		}


}
